#region Get-ConfigMgrSystemResourceList
function Get-ConfigMgrSystemResourceList
{
<#
.SYNOPSIS
	Getting System resources in hierarchy.
.DESCRIPTION
	Run query against SMS_SystemResourceList to retrieve list of Systems. It returns also
	roles so it's possible to query for specific role.
.EXAMPLE
	PS C:\> Get-ConfigMgrSystemResourceList -ComputerName server
	
	ServerName   SiteCode   RoleName                       NALPath
	----------   --------   --------                       -------
	server       tst        SMS Site Server                ["Display=\\server\"]MSWNET:["SMS_SITE=tst"]\\ser...
	server       tst        SMS Client Access Point        ["Display=\\server\"]MSWNET:["SMS_SITE=tst"]\\ser...
	server       tst        SMS Site System                ["Display=\\server\"]MSWNET:["SMS_SITE=tst"]\\ser...
	server       tst        SMS Component Server           ["Display=\\server\"]MSWNET:["SMS_SITE=tst"]\\ser...
	server       tst        SMS Management Point           ["Display=\\server\"]MSWNET:["SMS_SITE=tst"]\\ser...
	server       tst        SMS Distribution Point         ["Display=\\server\sms$\"]MSWNET:["SMS_SITE=tst"]...
	server       tst        SMS Site System                ["Display=\\server\sms$\"]MSWNET:["SMS_SITE=tst"]...

.LINK
	Get-ConfigMgrObject
#>

	[CmdletBinding()]
	
	param(
		# Server to look for
		[Parameter(
			Mandatory = $true,			
			ValueFromPipelineByPropertyName = $true,
			Position = 0,
			ParameterSetName = 'Name'
		)]
		[Alias('cn','Computer','Server', 'ServerName')]
		[string]
		$ComputerName = '%'
		,
		# Role of the server. 
		[Parameter()]
		[Alias('Role')]
		[ValidateSet('SMS Component Server', 'SMS Site Server',
			'SMS Site System', 'SMS SQL Server',
			'SMS Distribution Point', 'SMS Management Point',
			'SMS Reporting Point', 'SMS Server Locator Point',
			'SMS Software Update Point', 'SMS Fallback Status Point',
			'AI Update Service Point', 'SMS Client Access Point',
			'SMS Device Management Point'
		)]
		[string]
		$RoleName
		,
		# Use NALPath for searching.
		[Parameter(
			ParameterSetName = 'NAL'
		)]
		[Alias('ServerNALPath')]
		[string]
		$NALPath
	)
	
	BEGIN { }
	
	PROCESS
	{		
		switch ($PSCmdlet.ParameterSetName)
		{
			'Name'
				{
					Write-Verbose 'Get-ConfigMgrSystemResourceList/PSetName: Name'
					$ComputerName = $ComputerName -replace '\*', '%'
					
					if ($RoleName)
					{
						$filter = "ServerName LIKE '$ComputerName' AND RoleName = '$RoleName'"
						Write-Verbose $filter
					}
					else
					{
						$filter = "ServerName LIKE '$ComputerName'"
						Write-Verbose $filter
					}
					
					$cmParameters = @{
						Class  = 'SMS_SystemResourceList'
						Filter = $filter
					}
				} # 'Name'
			'NAL'
				{
					Write-Verbose 'Get-ConfigMgrSystemResourceList/PSetName: NALPath'
					if ($RoleName)
					{
						$filter = "NALPath = '$NALPath' AND RoleName = '$RoleName'"
						Write-Verbose $filter
					}
					else
					{
						$filter = "NALPath = '$NALPath'"
						Write-Verbose $filter
					}
					
					$cmParameters = @{
						Class  = 'SMS_SystemResourceList'
						Filter = $filter
					}
				} # 'NAL'
		} # switch

		Get-ConfigMgrObject @cmParameters
		
	} # PROCESS
	
	END { }
}
#endregion Get-ConfigMgrSystemResourceList

#region Get-ConfigMgrSite
function Get-ConfigMgrSite
{
<#
.SYNOPSIS
	Get site.
.DESCRIPTION
	Returns data related to the site.
.EXAMPLE
	PS C:\> Get-ConfigMgrSite -SiteCode tst
	
	ServerName     SiteCode ReportTo SiteName Version
	----------     -------- -------- -------- -------
	server         tst      PRI      My-Lab   4.00.1234.5000
	
.OUTPUTS
.LINK
	Get-ConfigMgrObject
#>

	[CmdletBinding(
		DefaultParameterSetName = 'Name'
	)]
	
	param(
		# Server to look for.
		[Parameter(
			Mandatory = $true,
			ValueFromPipeline = $true,
			ValueFromPipelineByPropertyName = $true,
			Position = 0,
			ParameterSetName = 'Name'
		)]
		[Alias('cn','Computer','Server')]
		[string[]]
		$ComputerName
		,
		# Site code to check.
		[Parameter(
			Position = 0,
			ParameterSetName = 'Site'
		)]
		[Alias('Site','sc')]
		[string]
		$SiteCode
	)
	
	BEGIN { }
	
	PROCESS
	{				
		switch ($PSCmdlet.ParameterSetName)
		{
			'Name'
				{
					$ComputerName = $ComputerName -replace '\*', '%'
					
					$cmParameters = @{
						Class  = 'SMS_Site'
						Filter = "ServerName LIKE '$ComputerName'"
					}
				} # 'Name'
				
			'Site'
				{
					$SiteCode = $SiteCode -replace '\*', '%'
					
					$cmParameters = @{
						Class  = 'SMS_Site'
						Filter = "SiteCode LIKE '$SiteCode'"
					}
				} # 'Site'
		} # switch
		
		Get-ConfigMgrObject @cmParameters
		
	} # PROCESS
	
	END { }
}
#endregion Get-ConfigMgrSite