#region Connect-ConfigMgrProvider
function Connect-ConfigMgrProvider
{
<#
.SYNOPSIS
	Connects to ConfigMgr provider.
	
.DESCRIPTION
	Connects to ConfigMgr and saves variable used by other functions from this module.
	
	Use this function whenever you want to use different computername or credentials.
	
.PARAMETER ComputerName
	Computer to connect.

.PARAMETER Credential
	Credentials to use for connection.
	
.EXAMPLE
	Connect-ConfigMgrProvider
		Connects to provider of local computer, will use your current credentials.
	
.EXAMPLE
	Connect-ConfigMgrProvider -ComputerName server -Credential domain\admin
		Connects to WMI provider of 'server', asks for password for 'domain\admin' account and
		use this account for connection to WMI.
	
.EXAMPLE
	Connect-ConfigMgrProvider -cn server -Credential $cred
		Connects to WMI provider of 'server' and use credentials stored in $cred variable
		
.LINK
	Disconnect-ConfigMgrProvider
#>
	[CmdletBinding()]
	
	param(
		[Parameter(
			Position = 0)]
		[ValidateNotNullOrEmpty()]
		[Alias('cn', 'co')]
		[string]
		$ComputerName = $env:COMPUTERNAME
		,
		[Parameter(
			Position = 1
		)]
		[Alias('cr')]
		[ValidateNotNull()]
		$Credential = [System.Management.Automation.PSCredential]::Empty
	)

	BEGIN 
	{
		Disconnect-ConfigMgrProvider
	}

	PROCESS
	{		
		Write-Verbose 'Inside PROCESS'
		try
		{
			Write-Verbose 'Checking if $Credential -is [string]'
			if ($Credential -is [string])
			{
				Write-Verbose '$Credential is String object'
				$cred = Get-Credential $Credential
			}
			else
			{
				Write-Verbose '$Credential is not String object'
				$cred = $Credential
			}
			
			Write-Verbose 'Binding $wmiParameters'
			$wmiParameters = @{
				Namespace    = 'root\sms'
				Class        = 'SMS_ProviderLocation'
				Filter       = 'ProviderForLocalSite="true"'
				ComputerName = $ComputerName
				Credential   = $cred
			}
			
			Write-Verbose 'Calling Get-WmiObject with @wmiParameters'
			$CMProviderLocation = Get-WmiObject @wmiParameters -ErrorAction Stop
			
			Write-Verbose 'Binding $prop'
			$prop = @{
				ComputerName      = $CMProviderLocation.Machine
				NamespacePath     = $CMProviderLocation.NamespacePath
				SiteCode          = $CMProviderLocation.SiteCode
				Namespace         = 'root\sms\site_' + $CMProviderLocation.SiteCode
				Credential        = $cred
			}			
			
			Write-Verbose 'Creating $Script:CMProvider object'
            $Script:CMProvider = New-Object -TypeName PSObject -Property $prop
			$Script:CMProvider.PSObject.TypeNames.Clear()
			$Script:CMProvider.PSObject.TypeNames.Insert(0, 'PSCMX.Provider')
			
			Read-ConfigMgrProviderSettings
		}
		catch [System.UnauthorizedAccessException]
		{
			Write-Verbose 'Exception: System.UnauthorizedAccessException'
			$err = $_
			Write-Host "$err" -ForegroundColor Red
			Write-Host " ==> Please provide correct username/password." -ForegroundColor Yellow
		}
		catch [System.SystemException]
		{
			Write-Verbose 'Exception: System.SystemException'
			$err = $_
			Write-Host "$err" -ForegroundColor Red
			#Write-Host " ==> Please provide correct computername." -ForegroundColor Yellow
			throw " ==> Please provide correct computername."
		}
		catch
		{
			Write-Verbose 'General "unhandled" exception'
		}
	}

	END 
	{ 
		if ($Global:PSCMXGenerateFormatFile)
		{
			$Script:NameToClass.Keys |% { 
				Write-Verbose "Name to Class: $_"
				$ffName = $PSScriptRoot + '\FormatData\PSCMX.Format.' + $_ + '.ps1xml'
				Write-Verbose "File name: $(Split-Path $ffName -Leaf)"
				
				if (-not (Test-Path $ffName))
				{
					$cmd = "New-$($_)FormatFile"
					Write-Verbose "iex command for new format file: $cmd"
					Invoke-Expression $cmd
				}
				else
				{
					Write-Verbose 'Updating format file.'
					Update-FormatFile $ffName
					Update-FormatData -AppendPath $ffName -ErrorAction SilentlyContinue
				}
			}
		}
	}	
}
#endregion Connect-ConfigMgrProvider

#region Read-ConfigMgrProviderSettings
function Read-ConfigMgrProviderSettings
{
<#
.SYNOPSIS
	Shows values currently used for connection to ConfigMgr server.
	
.DESCRIPTION
	Shows all properties currently used in module. Output is formatted by own format file.
	
.EXAMPLE
	Read-ConfigMgrProviderSettings
	Server      Sitecode   WMI Namespace        Full namespace path              Username
	------      --------   -------------        -------------------              --------
	CMSERVER    TST        root\sms\site_TST    \\CMSERVER\root\sms\site_TST
	
.LINK
	Connect-ConfigMgrProvider
#>
	
	$Script:CMProvider

}
#endregion Read-ConfigMgrProviderSettings

#region Get-ConfigMgrObject
function Get-ConfigMgrObject
{
<#
.SYNOPSIS
	Retrive WMI objects from ConfigMgr server namespace.

.DESCRIPTION
	This function is used by other functions for retrieving ConfigMgr WMI objects. It's possible
	to call it separately  if you need to receive some "non-standard" class not called by default
	with this module.

.PARAMETER Class
	Specify WMI class you want to retrieve.

.PARAMETER Filter
	Filter used to limit number of objects received from WMI.

.PARAMETER Query
	Complete WQL query used to retrieve WMI data.

.EXAMPLE
	Get-ConfigMgrObject -Class SMS_Site -Filter 'Type=2' | Select ServerName, SiteCode, SiteName | Out-GridView -Title 'Primary sites'
	Retrives all primary sites and shows server name, site code and site name in Grid.

.OUTPUTS
	System.Management.ManagementObject
#>
	[CmdletBinding(
		DefaultParameterSetName = 'Filter'
	)]
	
	param(		
		[Parameter(
			Mandatory = $true,
			Position = 0,
			ParameterSetName = 'Filter'
		)]
		[String]
		$Class
		,
		[Parameter(
			Position = 1,
			ParameterSetName = 'Filter'
		)]
		[String]
		$Filter
		,
		[Parameter(
			Mandatory = $true,
			ParameterSetName = 'Query'
		)]
		[string]
		$Query
	)

	BEGIN 
	{
		if (-not $Script:CMProvider)
		{
			Write-Verbose 'Calling Connect-ConfigMgrProvider'
			Connect-ConfigMgrProvider
		}
	}

	PROCESS
	{		
		Write-Verbose 'Inside PROCESS of Get-ConfigMgrObject function'
		
		Write-Verbose 'Binding $cmParameters' 
		$cmParameters = @{
			ComputerName = $Script:CMProvider.ComputerName
			Namespace    = $Script:CMProvider.Namespace
			Credential   = $Script:CMProvider.Credential
		}
				
		Write-Verbose "Calling Get-WmiObject with @cmParameters @PSBoundParameters"
		Get-WmiObject @cmParameters @PSBoundParameters				
	}

	END { }
}
#endregion Get-ConfigMgrObject

#region Set-ConfigMgrObject
function Set-ConfigMgrObject
{
	[CmdletBinding(
		DefaultParameterSetName = 'Class'
	)]
	
	param(
		[Parameter(
			Mandatory = $true,
			ParameterSetName = 'Class'
		)]
		[String]
		$Class
		,
		<#[Parameter(
			Mandatory = $true,
			ParameterSetName = 'Path'
		)]
		[String]
		$Path
		,#>
		[Parameter()]
		[hashtable]
		$Arguments
	)
	
	BEGIN {
		if (-not $Script:CMProvider)
		{
			Write-Verbose 'Calling Connect-ConfigMgrProvider'
			Connect-ConfigMgrProvider
		}
		
		$RejectAll = $false
		$ConfirmAll = $false
	}
	
	PROCESS
	{
		Write-Verbose 'Inside PROCESS of Set-ConfigMgrObject function'
		
		Write-Verbose 'Binding $cmParameters' 
		$cmParameters = @{
			ComputerName = $Script:CMProvider.ComputerName
			Namespace    = $Script:CMProvider.Namespace
			Credential   = $Script:CMProvider.Credential
		}
		
		Write-Verbose "Calling Set-WmiInstance with @cmParameters @PSBoundParameters"
		Set-WmiInstance @cmParameters @PSBoundParameters #-WhatIf
		#"Set-WmiInstance for`nClass: {0}`nPath: {1}" -f $Class, $Path
	}
	
	END { }
	
}
#endregion Set-ConfigMgrObject

#region Disconnect-ConfigMgrProvider
function Disconnect-ConfigMgrProvider
{
<#
.SYNOPSIS
	Removes variable from session.
	
.DESCRIPTION
	Removes internal module variable created by Connect-ConfigMgrPRovider.
	
	Use this function if you need to connect to other server or use different credentials.

.EXAMPLE
	Disconnect-ConfigMgrProvider
	Removes internal variable from module scope. For new connection to ConfigMgr namespace you have to
	call Connect-ConfigMgrProvider again.
	
.LINK
	Connect-ConfigMgrProvider
#>

	Remove-Variable -Name CMProvider -Scope Script -Force -ErrorAction SilentlyContinue

}
#endregion Disconnect-ConfigMgrProvider